#!/usr/bin/php
<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";

const CONFIG_FILE = "mgportal.conf";

require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP-base.php');

$Log = new Logger();
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
if ($Config->useAMQPLib()) {
    require_once(BASE_DIR . BIN_DIR . 'AMQPlib.php');
} else {
    require_once(BASE_DIR . BIN_DIR . 'AMQP.php');
}

$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile('to_device');

if ($argc == 3) {
    $manager = $Config->getFirstManager();
    if ($manager === null) {
        printf("Error:No manager is defined in server portion of mgportal.conf config file.");
        exit(255);
    }

    try{
        $device = $argv[1];
        $tag = $device . microtime(true);

        $AMQP = new AMQP($manager['host'], $Config->getUser($manager['host'], $device), $device);
        $AMQP->setPort($Config->getPort());
        $AMQP->setTimeout($Config->getTimeout());
        $AMQP->setWaitTimeout($Config->getWaitTimeout());
        $AMQP->connect($Config->useSSL());
        $channel = $AMQP->createChannel();
        $exchange = $AMQP->createExchange();

        $routing_key = $Config->generate_route('cloud', $device);
        $reply_key = $Config->generate_route_reply($routing_key);

        if ($Config->useAMQPLib()) {
            $queue = $AMQP->createQueue($routing_key, false);
            $q_reply_to = $AMQP->createQueue($reply_key);
        } else {
            $queue = new AMQPQueue($channel);
            $queue->setName($routing_key);

            try {
                $q_reply_to = new AMQPQueue($channel);
                $q_reply_to->setName($reply_key);
                $q_reply_to->declareQueue();
            }
            catch (Exception $ex) {
                $message = $ex->getMessage();
                $Log->logDebug("Reply to Queue exists " . $message);
            }
        }

        $message = $argv[2];

        if (!$Config->useAMQPLib()) {
            $exchange->publish($message, $routing_key, AMQP_NOPARAM, array('reply_to' => $reply_key));
            $Log->logDebug("Reply-to queue: " . $reply_key);

            $q_reply_to->consume(function (AMQPEnvelope $message, AMQPQueue $queue) use ($Log) {
                $response = $message->getBody();
                echo $response;
                $Log->logDebug('Response is ' . $response);
                $Log->logDebug('Received on ' . $queue->getName() . ' queue');
                return false;
            }, AMQP_AUTOACK, $tag); //, AMQP_JUST_CONSUME);

            $AMQP->disconnect();
        } else {
            $AMQP->publish($message, '', $routing_key, array('reply_to' => $reply_key));
            $Log->logDebug("Reply-to queue: " . $reply_key);

            $AMQP->consume(function ($msg) use ($AMQP, $Log, $reply_key) {
                $response = $msg->body;
                echo $response;
                $Log->logDebug('Response is ' . $response);
                $Log->logDebug('Received on ' . $reply_key . ' queue');
                $AMQP->disconnect();
                exit(0);
            }, $reply_key);

            $AMQP->waitLoop();
        }
    } catch(Exception $ex){
        $Log->logError($ex);
        $message = $ex->getMessage();
        if ($message == '(unknown error)') {
            $message = " Cannot contact appliance.  Please ensure it is online, this host is enabled for management, and the xg-portal service is running..";
        } else if (strstr($message, "could not connect to host")) {
            $message = " Cannot send message to remote appliance.  Please ensure that the messaging server is running on the local manager appliance.";
        } else {
            $waitTimeout = $Config->getWaitTimeout();
            if ($waitTimeout != null && strstr($message, "connection timed out after ")) {
                $message = "A timeout occurred while contacting appliance.  Please ensure it is online, this host is enabled for management, and the xg-portal service is running.";
            }
        }
        printf("Error:" . $message);
        if (isset($q_reply_to) && is_object($q_reply_to)) {
            $q_reply_to->cancel($tag);
        }
        exit(255);
    }   
} else {
    usage();
}

function usage() {
    printf("Invalid usage:\n");
    printf("    to_device device_id [command]\n\n");
}

?>