
SET search_path TO mgportal;

DO $$
BEGIN
	IF NOT EXISTS(SELECT schema_name from information_schema.schemata WHERE schema_name='mgportal')
	THEN
		EXECUTE 'CREATE SCHEMA mgportal';
	ELSE
		RAISE NOTICE 'mgportal schema exists';
	END IF;
END $$;

CREATE TABLE IF NOT EXISTS mgportal.managers (
  id SERIAL PRIMARY KEY,
  manager_id INTEGER DEFAULT 1 REFERENCES mgportal.managers (id) ON DELETE CASCADE,
  name VARCHAR NOT NULL,
  type SMALLINT DEFAULT 0,
  logoLight VARCHAR,
  logoDark VARCHAR,
  color VARCHAR
);

CREATE TABLE IF NOT EXISTS mgportal.locations (
  id SERIAL PRIMARY KEY,
  manager_id INTEGER DEFAULT 1 REFERENCES mgportal.managers (id) ON DELETE CASCADE,
  name VARCHAR NOT NULL,
  country VARCHAR,
  street VARCHAR,
  city VARCHAR,
  state VARCHAR,
  zip VARCHAR,
  phone VARCHAR,
  latitude VARCHAR,
  longitude VARCHAR
);

INSERT INTO mgportal.managers
(name)
  SELECT 'Default'
  WHERE
    NOT EXISTS (
        SELECT name FROM mgportal.managers WHERE name = 'Default'
    );

CREATE TABLE IF NOT EXISTS mgportal.manager_types(
  type INTEGER PRIMARY KEY,
  name VARCHAR
);

INSERT INTO mgportal.manager_types
(type, name)
    SELECT 0, 'Providers'
    WHERE
      NOT EXISTS (
        SELECT type FROM mgportal.manager_types WHERE type = 0
      );

INSERT INTO mgportal.manager_types
(type, name)
  SELECT 1, 'VARs'
  WHERE
    NOT EXISTS (
        SELECT type FROM mgportal.manager_types WHERE type = 1
    );

INSERT INTO mgportal.manager_types
(type, name)
  SELECT 2, 'Aggregators'
  WHERE
    NOT EXISTS (
        SELECT type FROM mgportal.manager_types WHERE type = 2
    );

INSERT INTO mgportal.manager_types
(type, name)
  SELECT 3, 'Distributors'
  WHERE
    NOT EXISTS (
        SELECT type FROM mgportal.manager_types WHERE type = 3
    );

INSERT INTO mgportal.manager_types
(type, name)
  SELECT 4, 'Companies'
  WHERE
    NOT EXISTS (
        SELECT type FROM mgportal.manager_types WHERE type = 4
    );

INSERT INTO mgportal.locations
(name)
  SELECT 'Default'
  WHERE
    NOT EXISTS (
        SELECT name FROM mgportal.locations WHERE name = 'Default'
    );


CREATE TABLE IF NOT EXISTS mgportal.users (
  user_id SERIAL PRIMARY KEY,
  username VARCHAR(128) UNIQUE,
  password VARCHAR(256) NOT NULL,
  privileges INTEGER DEFAULT 0,
  email_address VARCHAR,
  company_id INTEGER DEFAULT 1 NOT NULL REFERENCES mgportal.managers(id) ON DELETE CASCADE
);

INSERT INTO mgportal.users
(username, password, privileges)
    SELECT 'root','$2y$10$9l7AulbwWISyh38hGGL.tOF4HNlDY5W3PXeQmtQyRDGJr4.H1HWjq',3
    WHERE
      NOT EXISTS (
        SELECT username FROM mgportal.users WHERE username = 'root'
      );

CREATE TABLE IF NOT EXISTS mgportal.privileges (
  level INTEGER PRIMARY KEY,
  name VARCHAR
);

INSERT INTO mgportal.privileges
(level, name)
  SELECT 0, 'monitor'
  WHERE
    NOT EXISTS (
        SELECT level FROM mgportal.privileges WHERE level = 0
    );

INSERT INTO mgportal.privileges
(level, name)
  SELECT 1, 'manage'
  WHERE
    NOT EXISTS (
        SELECT level FROM mgportal.privileges WHERE level = 1
    );

INSERT INTO mgportal.privileges
(level, name)
  SELECT 2, 'administrator'
  WHERE
    NOT EXISTS (
        SELECT level FROM mgportal.privileges WHERE level = 2
    );

INSERT INTO mgportal.privileges
(level, name)
  SELECT 3, 'superuser'
  WHERE
    NOT EXISTS (
        SELECT level FROM mgportal.privileges WHERE level = 3
    );


CREATE TABLE IF NOT EXISTS mgportal.sessions (
  userid INTEGER NOT NULL,
  uuid VARCHAR NOT NULL,
  expiry BIGINT NOT NULL
);


CREATE TABLE IF NOT EXISTS mgportal.devices (
  id SERIAL PRIMARY KEY,
  unique_identifier VARCHAR NOT NULL,
  hostname VARCHAR,
  has_support_contract BOOLEAN DEFAULT FALSE,
  license_class VARCHAR DEFAULT NULL,
  license_mkt VARCHAR DEFAULT NULL,
  license_features VARCHAR DEFAULT NULL,
  install_date VARCHAR NOT NULL,
  date_created BIGINT NOT NULL,
  location_id INTEGER DEFAULT 1 NOT NULL REFERENCES mgportal.locations(id) ON DELETE CASCADE,
  activity SMALLINT DEFAULT 0
);

CREATE TABLE IF NOT EXISTS mgportal.protected (
  id SERIAL PRIMARY KEY,
  date BIGINT,
  vm_count INTEGER DEFAULT 0,
  server_count INTEGER DEFAULT 0,
  app_count INTEGER DEFAULT 0,
  gb_used REAL DEFAULT 0,
  vm_replicated INTEGER DEFAULT 0,
  all_replicated INTEGER DEFAULT 0,
  device_id INTEGER NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS mgportal.monthly (
  id SERIAL PRIMARY KEY,
  date BIGINT,
  protected_assets_by_query INTEGER DEFAULT 0,
  protected_assets_by_api INTEGER DEFAULT 0,
  device_id INTEGER NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS mgportal.assets (
  device_id INT NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE,
  asset_type VARCHAR,
  client_name VARCHAR,
  instance_name VARCHAR,
  system_name VARCHAR,
  protected BOOL,
  protected_by_backup_copy BOOL
);

CREATE TABLE IF NOT EXISTS mgportal.backups (
  device_id INT NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE,
  app_name  VARCHAR,
  certify_status  VARCHAR,
  client_name VARCHAR,
  complete SMALLINT,
  database_name VARCHAR,
  elapsed_time VARCHAR,
  encrypted SMALLINT,
  end_date VARCHAR,
  files INT,
  id INT,
  instance_name VARCHAR,
  replication_status VARCHAR,
  size INT,
  start_date VARCHAR,
  status VARCHAR,
  system_name VARCHAR,
  type VARCHAR,
  verify_status VARCHAR
);

CREATE TABLE IF NOT EXISTS mgportal.copies (
  device_id INT NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE,
  app_name  VARCHAR,
  client_name VARCHAR,
  complete SMALLINT,
  database_name VARCHAR,
  elapsed_time VARCHAR,
  encrypted SMALLINT,
  end_date VARCHAR,
  files INT,
  id INT,
  instance_name VARCHAR,
  size INT,
  start_date VARCHAR,
  status VARCHAR,
  system_name VARCHAR,
  type VARCHAR
);

CREATE TABLE IF NOT EXISTS mgportal.alerts (
  device_id INT NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE,
  created VARCHAR,
  id INT,
  message VARCHAR,
  resolved SMALLINT,
  severity VARCHAR,
  sname VARCHAR,
  source_id INT,
  source_name VARCHAR,
  system_id INT,
  updated VARCHAR
);

CREATE TABLE IF NOT EXISTS mgportal.jobs (
  device_id INT NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE,
  app_id INT,
  application_name VARCHAR,
  calendar_str VARCHAR,
  description VARCHAR,
  enabled INT,
  id VARCHAR,
  last_status VARCHAR,
  last_time VARCHAR,
  name VARCHAR,
  next_time VARCHAR,
  sid INT,
  status VARCHAR,
  system_name VARCHAR,
  type VARCHAR

);

CREATE TABLE IF NOT EXISTS mgportal.timestamps (
  device_id INT NOT NULL REFERENCES mgportal.devices(id) ON DELETE CASCADE,
  subsystem INT, -- 0 - backups, 1 - copies - 2 - alerts, 3 - assets, 4 - jobs
  update_time BIGINT NOT NULL,
  day_count SMALLINT
);

CREATE TABLE IF NOT EXISTS mgportal.preferences (
  username VARCHAR NOT NULL,
  key VARCHAR NOT NULL,
  value VARCHAR NOT NULL
);

CREATE TABLE IF NOT EXISTS mgportal.credentials (
  credential_id SERIAL PRIMARY KEY NOT NULL,
  username VARCHAR(128) NOT NULL,
  password VARCHAR(256) NOT NULL,
  domain VARCHAR,
  display_name VARCHAR,
  is_default BOOLEAN DEFAULT FALSE
);