<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class Assets extends BaseAPI {
    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($which = null){
        $query = "SELECT * FROM mgportal.assets";
        if ($which != null) {
            $query .= " WHERE device_id=" . $which;
        }

        $asset = array();
        $assets = array();

        $dbAssets = Database::db_query($this->conn, $this->Log, $query);
        if ($dbAssets !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbAssets)){
                $stuff = $row[0];

                $asset = array('stuff' => $stuff);
                $assets[] = $asset;
            }
        }
        return array("assets" => $assets);
    }
    
    public function addAsset($data, $clearFirst){
        if (!isset($data["device_id"])){
            $message = "No device_id in data.";
            $this->Log->logInfo($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        } else {
            $device_id = $data['device_id'];
        }

        if (!is_array($data)) {
            $message = "Data not provided / empty";
            $this->Log->logInfo($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        } else {
            // check to see if clear first.
            if ($clearFirst) {
                $result = $this->clearAssets($device_id, true);
                if ($result !== true) {
                    $result = array(
                        'result' => false,
                        'message' => 'Error clearing previous assets.'
                    );
                    return $result;
                }
            }
            // build query
            $ins_query = "INSERT INTO mgportal.assets (device_id";
            $add_val_query = ") VALUES (%d";
            $val_query = sprintf($add_val_query, $data['device_id']);
            if (isset($data['asset_type'])) {
                $ins_query .= ", asset_type";
                $add_val_query = ", '%s'";
                $val_query .= sprintf($add_val_query, $data['asset_type']);
            }
            if (isset($data['client_name'])) {
                $ins_query .= ", client_name";
                $add_val_query = ", %s";
                $val_query .= sprintf($add_val_query, Database::db_escape_string($data['client_name']));
            }
            if (isset($data['instance_name'])) {
                $ins_query .= ", instance_name";
                $add_val_query = ", %s";
                $val_query .= sprintf($add_val_query, Database::db_escape_string($data['instance_name']));
            }
            if (isset($data['system_name'])) {
                $ins_query .= ", system_name";
                $add_val_query = ", '%s'";
                $val_query .= sprintf($add_val_query, $data['system_name']);
            }
            if (isset($data['protected'])) {
                $ins_query .= ", protected";
                $add_val_query = ", %b";
                $val_query .= sprintf($add_val_query, $data['protected']);
            }
            if (isset($data['protected_by_backup_copy'])) {
                $ins_query .= ", protected_by_backup_copy";
                $add_val_query = ", %b";
                $val_query .= sprintf($add_val_query, $data['protected_by_backup_copy']);
            }
            $val_query .= ");";
            $query = $ins_query . $val_query;

            $result = Database::db_query($this->conn, $this->Log, $query);
            return $result;
        }

    }


    public function clearAssets($deviceId, $okayIfNoMatch = false){
        $query = "DELETE FROM mgportal.assets";
        $query .= " WHERE device_id=" . $deviceId;


        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $result);
            if ($row == null && !$okayIfNoMatch){
                $result = array(
                    'result' => false,
                    'message' => "Error, doesn't exist"
                );
            } else {
                $result = true;
            }
        } else {
            $result = array(
                'result' => false,
                'message' => "Error deleting assets"
            );
        }
        return $result;
    }
}

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $assets = new Assets($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $result = $assets->get($which);
                $http->buildResult($result, "get");
                break;
            case 'post':
                $clearPriorItems = (isset($_GET['refresh']) && $_GET['refresh'] == 1);
                $result = $assets->addAsset($http->getData(), $clearPriorItems);
                $http->buildResult($result, "post");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}