<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class BackupCopies extends BaseAPI {

    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($which, $dow){
        $query = "SELECT device_id, app_name, client_name, instance_name, database_name, elapsed_time, start_date, end_date, status, complete, type, size, encrypted, id, system_name FROM mgportal.copies";
        if ($which !== null) {
            $query .= " WHERE device_id=" . $which;
        }

        $backups = array();

        $dbBackups = Database::db_query($this->conn, $this->Log, $query);
        if ($dbBackups !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbBackups)){
                $backup = array(    'device_id' => $row[0],
                    'application_name' => ($row[1] == "") ? "file-level" : $row[1],
                    'client_name' => $row[2],
                    'instance_name' => $row[3],
                    'database_name' => $row[4],
                    'elapsed_time' => $row[5],
                    'start_date' => $row[6],
                    'end_date' => $row[7],
                    'status' => $row[8],
                    'complete' => $row[9] == 1,
                    'type' => $row[10],
                    'size' => $row[11],
                    'encrypted' => $row[12] == 1,
                    'id' => $row[13],
                    'system_name' => $row[14]);

                if ($backup['elapsed_time'] == '') {
                    $backup['elapsed_time'] = 'Not Available';
                }
                
                if ($dow != null) {
                    // If day of week specified, see if backup start date matches.
                    $dayOfWeek = date('w', strtotime($backup['start_date']));
                    if ($dayOfWeek != $dow) {
                        continue;
                    }
                }

                $backups[] = $backup;
            }
        }
        return array("backups" => $backups);
    }

    public function addReplication($data, $clearFirst = false){
        $result = false;
        $device_id = -1;
        if (!$this->validateAsset($data, $result, $device_id)) {
            return $result;
        }

        if (!is_array($data['data'])) {
            $message = "Data not provided / empty";
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        } else {
            // check to see if clear first.
            if ($clearFirst) {
                $result = $this->clearBackups($device_id, true);
                if ($result !== true) {
                    $result = array(
                        'result' => false,
                        'message' => 'Error clearing previous backups.'
                    );
                    return $result;
                }
            }
            // insert into database
            $fulldata = $data['data'];
            $timestamp = $data['meta_timestamp'];
            $days = isset($data['data_days']) ? $data['data_days'] : 1;
            foreach ($fulldata as $data) {
                if (!is_array($data)) {
                    // do nothing with this one
                    $message = "Data not provided / empty";
                    $this->Log->logInfo($message);
                    $result = array(
                        'result' => false,
                        'message' => $message
                    );
                    // if array - continue
                } else {
                    // build query
                    $app_name = isset($data['app_name']) ? $data['app_name'] : null;
                    $client_name = isset($data['client_name']) ? $data['client_name'] : null;
                    $complete = isset($data['complete']) ? $data['complete'] : null;
                    $database_name = isset($data['database_name']) ? $data['database_name'] : null;
                    $elapsed_time = isset($data['replication_elapsed_time']) ? $data['replication_elapsed_time'] : null;
                    $encrypted = isset($data['encrypted']) ? $data['encrypted'] : null;
                    $end_date = isset($data['replication_end_date']) ? $data['replication_end_date'] : null;
                    $files = isset($data['files']) ? $data['files'] : null;
                    $id = isset($data['id']) ? $data['id'] : null;
                    $instance_name = isset($data['instance_name']) ? $data['instance_name'] : null;
                    $replication_status = isset($data['replication_status']) ? $data['replication_status'] : null;
                    if ($replication_status == 'Completed') {
                        // Convert replication status to same string as other statuses.
                        $replication_status = "Success";
                    }
                    $size = isset($data['size']) ? $data['size'] : null;
                    $start_date = isset($data['replication_start_date']) ? $data['replication_start_date'] : null;
                    $status = isset($data['status']) ? $data['status'] : null;
                    $system_name = isset($data['system_name']) ? $data['system_name'] : null;
                    $type = isset($data['type']) ? $data['type'] : null;


                    $ins_query = "INSERT INTO mgportal.copies (device_id, app_name, client_name, complete";
                    $ins_query .= ", database_name, elapsed_time, encrypted, end_date, files, id, instance_name";
                    $ins_query .= ", size, start_date, status, system_name, type) ";

                    $add_val_query = "VALUES (%d, '%s', %s, %d, %s, '%s', %d, '%s', %d, %d, %s, ";
                    $add_val_query .=        " %d, '%s', '%s', '%s', '%s');";
                    $val_query = sprintf($add_val_query, $device_id, $app_name,
                                        Database::db_escape_string($client_name), $complete,
                                        Database::db_escape_string($database_name), $elapsed_time, $encrypted, $end_date, $files, $id,
                                        Database::db_escape_string($instance_name),
                                        $size, $start_date, $replication_status, $system_name, $type);

                    $query = $ins_query . $val_query;

                    $queryResult = Database::db_query($this->conn, $this->Log, $query);
                    if ($queryResult === false) {
                        $this->Log->logError("Query returned false -- addBackups");
                        $result = false;
                    }
                }
            }
            $this->setTimestamp('copies', $timestamp, $days, $device_id);
        }
        return $result;
    }

    public function clearBackups($deviceId, $okayIfNoMatch = false){
        if ($deviceId === null) {
            $result = array(
                'result' => false,
                'message' => "No device present for clearing prior backup copies"
            );
            return $result;
        }
        $query = "DELETE FROM mgportal.copies";
        $query .= " WHERE device_id=" . $deviceId;

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $result);
            if ($row == null && !$okayIfNoMatch){
                $result = array(
                    'result' => false,
                    'message' => "Error, doesn't exist"
                );
            } else {
                $result = true;
            }
        } else {
            $result = array(
                'result' => false,
                'message' => "Error deleting backups"
            );
        }
        return $result;
    }

    public function addReplicationSummary($data, $clearPriorItems) {
        $result = false;
        $device_id = -1;
        if (!$this->validateAsset($data, $result, $device_id)) {
            return $result;
        }

        return true;
    }


};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $backups = new BackupCopies($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        $dow = isset($_GET['dow']) ? $_GET['dow'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $result = $backups->get($which, $dow);
                $http->buildResult($result, "get");
                break;
            case 'post':
                $clearPriorItems = (isset($_GET['refresh']) && $_GET['refresh'] == 1);
                $summaryInfo = (isset($_GET['summary']) && $_GET['summary'] == 1);
                if ($summaryInfo) {
                    $result = $backups->addReplicationSummary($http->getData(), $clearPriorItems);
                } else {
                    $result = $backups->addReplication($http->getData(), $clearPriorItems);
                }
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $backups->clearBackups($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}

?>