<?php
/**
 *  Credentials API
 */


const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");
require_once("./system/" . "password.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Credentials extends BaseAPI {
    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    public function get($which, $returnPassword = false, $returnDevicesUsingCredentials = false) {

        $query = "SELECT credentials.credential_id, credentials.username, credentials.password, credentials.domain, credentials.display_name, credentials.is_default FROM mgportal.credentials";
        if ($which !== null) {
            $query .= " WHERE credential_id=" . $which;
        }
        $query .= ";";
        $credentials = array();

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $result)) {
                $id = $row[0];
                $userName = $row[1];
                $encodedPassword = $row[2];
                $domain = $row[3] !== null ? $row[3] : "";
                $display_name = $row[4] !== null && $row[4] !== "" ? $row[4] : self::UNNAMED_CREDENTIALS;
                $is_default = $row[5] === 't' ? true : false;
                $credential = array('credential_id' => $id,
                              'username' => $userName,
                              'domain' => $domain,
                              'display_name' => $display_name,
                              'is_default' => $is_default);
                if ($returnPassword) {
                    $credential['password'] = base64_decode($encodedPassword);
                }
                if ($returnDevicesUsingCredentials) {
                    $credential['hosts'] = $this->getDevicesUsingCredential($id);
                }
                $credentials[] = $credential;
            }
        }
        return array("credentials" => $credentials);
    }

    /*
     * Given a credential ID, returns the hostnames of devices using this credential.
     */
    private function getDevicesUsingCredential($credential_id) {
        $hosts = array();
        $query = "SELECT hostname FROM mgportal.devices WHERE credential_id=" . $credential_id . ';';
        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $result)) {
                $hosts[] = $row[0];
            }
        }
        $hostsString = implode(",", $hosts);
        return $hostsString;
    }

    /*
     * Returns true if the credential by name exists already.
     */
    private function credentialNameExists($credentialName, $exclude_credential_id = null) {
        $found = false;
        $query = "SELECT credential_id, display_name FROM mgportal.credentials WHERE display_name='" . $credentialName . "'";
        if ($exclude_credential_id !== null) {
            $query .= " AND credential_id != " . $exclude_credential_id;
        }
        $query .= ";";
        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false) {
            $found = Database::db_num_rows($this->Log, $result) > 0;
        }
        return $found;
    }

    private function addCredential($userName, $password, $domain = null, $display_name = null, $is_default = false) {
        $credential_id = false;
        global $Log;

        if ($this->credentialNameExists($display_name)) {
            $credential_id = array(
                'result' => false,
                'message' => 'The name of the credentials must be unique.'
            );
        } else {
            $encodedPassword = base64_encode($password);
            $query = "INSERT INTO mgportal.credentials(username, password";
            $values = " VALUES ('" . $userName . "', '" . $encodedPassword . "'";

            if ($domain != null) {
                $query .= ", domain";
                $values .= ",'" . $domain . "'";
            }
            if ($display_name != null) {
                $query .= ", display_name";
                $values .= ", " . Database::db_escape_string($display_name);
            }
            if ($is_default === true) {
                $query .= ", is_default";
                $values .= ", 1";
            }

            $query .= ")";
            $values .= ")";

            $query .= $values . " RETURNING credential_id";
            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false) {
                $row = Database::db_fetch_row($this->Log,  $result);
                $credential_id = $row[0];
            } else {
                $Log->logError("Error saving credential information");
            }
        }
        return $credential_id;
    }

    public function updateCredential($credential_id, $options) {
        $result = false;

        if ($credential_id === null) {
            $result = array(
                'result' => false,
                'message' => 'Credential id is not optional for update.'
            );
        } else if (!$this->credentialExists($credential_id)) {
            $result = array(
                'result' => false,
                'message' => 'The credentials with id ' . $credential_id . ' do not exist.'
            );
        } else if (isset($options['display_name']) && $this->credentialNameExists($options['display_name'], $credential_id)) {
            $result = array(
                'result' => false,
                'message' => 'The name of the credentials must be unique.'
            );
        } else {
            if (is_array($options)) {
                $query = "UPDATE mgportal.credentials SET ";
                if (isset($options['password'])) {
                    $password = $options['password'];
                    $encodedPassword = base64_encode($password);
                    $query .= "password='" . $encodedPassword . "'";
                }
                if (isset($options['username'])) {
                    if (isset($password)) {
                        $query .= ", ";
                    }
                    $username = $options['username'];
                    $query .= "username='" . $username . "'";
                }
                if (isset($options['domain'])){
                    if (isset($password) || isset($username)){
                        $query .= ", ";
                    }
                    $domain = $options['domain'];
                    $query .= "domain='" . $domain . "'";
                }
                if (isset($options['display_name'])){
                    if (isset($password) || isset($username) || isset($domain)){
                        $query .= ", ";
                    }
                    $displayName = $options['display_name'];
                    $query .= "display_name=" . Database::db_escape_string($displayName);
                }

                $query .= " WHERE credential_id=" . $credential_id . ";";
                //var_dump($query);
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $result = true;
                } else {
                    $this->Log->logError("Error saving credential information");
                }
            }          
        }
        return $result;
    }

    private function credentialExists($id) {
        $result = false;
        $credentialsArray = $this->get($id);
        if (isset($credentialsArray['credentials'])) {
            $credentialsList = $credentialsArray['credentials'];
            if (count($credentialsList) > 0) {
                $result = true;
            }
        }
        return $result;
    }

    /*
     * Associates an existing credential by ID to the device.  If the credential ID is 0, set the device's credential to NULL.
     */
    public function associateDevice($credential_id, $device_id) {
        $credential_id = (int)$credential_id;
        if ($credential_id === null) {
            $result = array(
                'result' => false,
                'message' => 'Credential id is not optional for association.'
            );
        } else if ($device_id === null) {
            $result = array(
                'result' => false,
                'message' => 'Device id is not optional for association.'
            );
        } else if ($credential_id !== 0 && !$this->credentialExists($credential_id)) {
            $result = array(
                'result' => false,
                'message' => 'The credentials with id ' . $credential_id . ' do not exist.'
            );
        } else {
            // If credential ID is 0, set device credential to NULL.
            $credQuery = ((int)$credential_id === 0) ? "NULL" : $credential_id;
            $query = "UPDATE mgportal.devices SET credential_id=" . $credQuery . " WHERE id=" . $device_id . ";";

            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false) {
                $result = true;
            } else {
                $this->Log->logError("Error associating credential with device.");
                $result = array(
                    'result' => false,
                    'message' => 'Error associating credential with device.'
                );
            }
        }
        return $result;
    }

    public function post($data) {
        $userName = $data['username'];
        $password = $data['password'];
        $domain = isset($data['domain']) ? $data['domain'] : null;
        $displayName = isset($data['display_name']) ? $data['display_name'] : null;
        $isDefault = isset($data['is_default']) ? $data['is_default'] : 0;

        $result = $this->addCredential($userName, $password, $domain, $displayName, $isDefault);

        return $result;
    }

    public function delete($credential_id) {
        if ($credential_id === null) {
            $result = array(
                'result' => false,
                'message' => 'Credential id is not optional for delete.'
            );
        } else {
            $query = "DELETE FROM mgportal.credentials WHERE credential_id=";
            $query .= $credential_id;
            $query .= " RETURNING credential_id;";
            $query .= ";";
            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false) {
                $row = Database::db_fetch_row($this->Log,  $result);
                if ($row == null) {
                    $result = array(
                        'result' => false,
                        'message' => "Error: credential id " . $credential_id . " does not exist"
                    );
                } else {
                    $result = true;
                }
            } else {
                $message = "Error deleting credential " . $credential_id;
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
        }
        return $result;
    }
};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $credentials = new Credentials($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $returnPassword = isset($_GET['pw']) ? (int)$_GET['pw'] === 1 : false;
                $returnDevices = isset($_GET['devices']) ? (int)$_GET['devices'] === 1 : false;
                $result = $credentials->get($which, $returnPassword, $returnDevices);
                $http->buildResult($result, "get");
                break;
            case 'put':
                $associatedDevice = isset($_GET['device_id']) ? (int)$_GET['device_id'] : 0;
                if ($associatedDevice > 0) {
                    // associate an existing credential with a device
                    $result = $credentials->associateDevice($which, $associatedDevice);
                } else {
                    $result = $credentials->updateCredential($which, $http->getData());
                }
                $http->buildResult($result, "put");
                break;
            case 'post':
                $result = $credentials->post($http->getData());
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $credentials->delete($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}
?>