<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class Jobs extends BaseAPI {

    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($which){
        $query = "SELECT device_id, app_id, application_name, name, last_time, next_time, status, type, description, enabled, id, system_name, sid FROM mgportal.jobs";
        if ($which !== null) {
            $query .= " WHERE device_id=" . $which;
        }

        $jobs = array();

        $dbJobs = Database::db_query($this->conn, $this->Log, $query);
        if ($dbJobs !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbJobs)){
                $job = array(   'device_id' => $row[0],
                    'app_id' => $row[1],
                    'application_name' => $row[2],
                    'name' => $row[3],
                    'last_time' => $row[4],
                    'next_time' => $row[5],
                    'status' => $row[6],
                    'type' => $row[7],
                    'description' => $row[8],
                    'enabled' => $row[9] == 1,
                    'id' => $row[10],
                    'system_name' => $row[11],
                    'sid' => $row[12]);
                $jobs[] = $job;
            }
        }
        return array("jobs" => $jobs);
    }

    public function addJobs($data, $clearFirst){
        global $Log;
        $result = false;
        $device_id = -1;
        if (!$this->validateAsset($data, $result, $device_id)) {
            return $result;
        }

        if (!is_array($data['data'])) {
            $message = "Data not provided / empty";
            $Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        } else {
            // check to see if clear first.
            if ($clearFirst) {
                $result = $this->clearJobs($device_id, true);
                if ($result !== true) {
                    $result = array(
                        'result' => false,
                        'message' => 'Error clearing previous jobs.'
                    );
                    return $result;
                }
            }
            // insert into database
            $fulldata = $data['data'];
            $timestamp = $data['meta_timestamp'];
            foreach ($fulldata as $data) {
                // check if array
                if (!is_array($data)) {
                    // do nothing with this one
                    $message = "Data not provided / empty";
                    $Log->logInfo($message);
                    $result = array(
                        'result' => false,
                        'message' => $message
                    );
                    // if array - continue
                } else {
                    // build query
                    $app_id = isset($data['app_id']) ? $data['app_id'] : null;
                    $application_name = isset($data['application_name']) ? $data['application_name'] : null;
                    $calendar_str = isset($data['calendar_str']) ? $data['calendar_str'] : null;
                    if (is_array($calendar_str)){
                        $cal_str = $calendar_str;
                        $calendar_str = "";
                        foreach ($cal_str as $string){
                            $calendar_str .= $string . ",";
                        }
                    }
                    $description = isset($data['description']) ? $data['description'] : null;
                    $enabled = isset($data['enabled']) ? $data['enabled'] : null;
                    $id = isset($data['id']) ? $data['id'] : null;
                    $last_status = isset($data['last_status']) ? $data['last_status'] : null;
                    $last_time = isset($data['last_time']) ? $data['last_time'] : null;
                    $name = isset($data['name']) ? $data['name'] : null;
                    $next_time = isset($data['next_time']) ? $data['next_time'] : null;
                    $sid = isset($data['sid']) ? $data['sid'] : null;
                    $status = isset($data['status']) ? $data['status'] : null;
                    $system_name = isset($data['system_name']) ? $data['system_name'] : null;
                    $type = isset($data['type']) ? $data['type'] : null;


                    $ins_query = "INSERT INTO mgportal.jobs (device_id, app_id, application_name, calendar_str";
                    $ins_query .= ", description, enabled, id, last_status, last_time, name, next_time, sid, status, system_name, type)";

                    $add_val_query = " VALUES (%d, %d, '%s', '%s', '%s', %d, '%s', '%s', '%s', '%s', '%s', %d, '%s', '%s', '%s');";
                    $val_query = sprintf($add_val_query, $device_id, $app_id, $application_name, $calendar_str, $description,
                        $enabled, $id, $last_status, $last_time, $name, $next_time, $sid,
                        $status, $system_name, $type);

                    $query = $ins_query . $val_query;

                    $queryResult = Database::db_query($this->conn, $this->Log, $query);
                    if ($queryResult === false) {
                        $Log->logError("Query returned false -- addAlerts");
                        $result = array(
                            'result' => false,
                            'message' => "Error adding job information for device: " . $device_id
                        );
                        break;
                    }
                }
            }
            $this->setTimestamp('jobs', $timestamp, 0, $device_id);
        }
        return $result;
    }

    public function clearJobs($deviceId, $okayIfNoMatch = false){
        if ($deviceId === null) {
            $result = array(
                'result' => false,
                'message' => "No device present for clearing prior jobs"
            );
            return $result;
        }
        $query = "DELETE FROM mgportal.jobs";
        $query .= " WHERE device_id=" . $deviceId;

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $result);
            if ($row == null && !$okayIfNoMatch){
                $result = array(
                    'result' => false,
                    'message' => "Error, doesn't exist"
                );
            } else {
                $result = true;
            }
        } else {
            $result = array(
                'result' => false,
                'message' => "Error deleting jobs"
            );
        }
        return $result;
    }


};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $jobs = new Jobs($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $result = $jobs->get($which);
                //$result = $users->forgotPassword(1);
                $http->buildResult($result, "get");
                break;
            case 'post':
                $clearPriorItems = (isset($_GET['refresh']) && $_GET['refresh'] == 1);
                $result = $jobs->addJobs($http->getData(), $clearPriorItems);
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $jobs->clearJobs($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}




?>
