<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger();
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Managers extends BaseAPI {

    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    public function get($which, $name) {
        $query = "SELECT id, manager_id, name, type, logolight, logodark, color FROM mgportal.managers";
        if ($which !== null) {
            $query .= " WHERE id=" . $which;
        } else if ($name !== null) {
            $query .= " WHERE name='" . $name . "'";
        }
        $query .= ";";
        $managers = array();

        $dbManagers = Database::db_query($this->conn, $this->Log, $query);
        if ($dbManagers !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbManagers)) {
                //printf("asset = %s\n", $row[1]);
                $id = $row[0];
                $manager_id = $row[1];
                $name = $row[2];
                $type = $row[3];
                $logolight = $row[4];
                $logodark = $row[5];
                $color = $row[6];

                $manager = array('id' => $id,
                    'manager_id' => $manager_id,
                    'name' => $name,
                    'type' => $type,
                    'logolight' => $logolight,
                    'logodark' => $logodark,
                    'color' => $color);
                $managers[] = $manager;
            }
        }
        return array("managers" => $managers);
    }
    
    public function getByType($type){
        $query = "SELECT id, manager_id, name, country, street, city, state, zip, phone, type FROM mgportal.managers";
        $query .= " WHERE type=" . $type . ";";

        $managers = array();
        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            if ($row = Database::db_fetch_row($this->Log,  $result)) {
                $id = $row[0];
                $manager_id = $row[1];
                $name = $row[2];
                $country = $row[3];
                $street = $row[4];
                $city = $row[5];
                $state = $row[6];
                $zip = $row[7];
                $phone = $row[8];
                $type = $row[9];
                $manager = array('id' => $id, 'manager_id' => $manager_id, 'name' => $name,
                                 'country' => $country, 'street' => $street, 'city' => $city,
                                 'state' => $state, 'zip' => $zip, 'phone' => $phone, 'type' => $type);
                $managers[] = $manager;
            } else {
                $managers = false;
            }
        }
        return array("managers" => $managers);
    }
//SELECT managers.type FROM mgportal.managers JOIN mgportal.manager_types on (managers.type = manager_types.type) WHERE id=2;

    public function getManagerType($id, $text = false){
        $query = "SELECT managers.type";
        if ($text) $query .= ", manager_types.name";
        $query .= " FROM mgportal.managers";
        if ($text) $query .= " JOIN mgportal.manager_types on (managers.type = manager_types.type)";
        $query .= " WHERE id=" . $id . ";";

        $type = $name = false;
        $dbType = Database::db_query($this->conn, $this->Log, $query);
        if ($dbType !== false){
            if ($row = Database::db_fetch_row($this->Log,  $dbType)){
                $type = $row[0];
                if ($text) $name = $row[1];
            }
        }
        if ($text){
            return array("type" => $type, "text" => $name);
        } else {
            return array("type" => $type);
        }
    }

    public function getChildren($id){
        global $Log;
        $children = array();

        $query = "SELECT id, name, type FROM mgportal.managers";
        if ($id != null) {
            $query .= " WHERE manager_id=" . $id . ";";
        }

        $dbManagers = Database::db_query($this->conn, $this->Log, $query);
        if ($dbManagers !== false){
            if(Database::db_num_rows($this->Log, $dbManagers) < 1){
                return false;
            }
            while ($row = Database::db_fetch_row($this->Log,  $dbManagers)){
                $myID = $row[0];
                $name = $row[1];
                $type = $row[2];
                $child = array('id' => $myID, 'name' => $name, 'type' => $type, 'parent_id' => $id);
                $children[] = $child;
            }
        }

        return array("children" => $children);
    }
    
    public function getAllChildren($parentId = 0)
    {
        $branch = array();

        $kids = $this->getChildren($parentId);
        if ($kids !== false && is_array($kids['children'])) {
            $kids = $kids['children'];
            foreach ($kids as $kid) {
                if ($kid['id'] != 1) {
                    $children = $this->getAllChildren($kid['id']);
                    if ($children) {
                        $kid['children'] = $children;
                    }
                    $branch[] = $kid;
                }
            }
        }

        return $branch;
    }

    public function updateManager($id, $options) {
        $result = false;

        if ($id === null) {
            $result = array(
                'result' => false,
                'message' => 'This id is not optional for update.'
            );
        } else {
            if (is_array($options)) {
                $query = "UPDATE mgportal.managers ";

                $query .= "SET ";
                if (isset($options['name'])) {
                    $query .= "name=";
                    $query .= Database::db_escape_string($options['name']);
                    $query .= ",";
                }
                if (isset($options['manager_id'])) {
                    $query .= "manager_id=";
                    $query .= $options['manager_id'];
                    $query .= ",";
                }
                if (isset($options['type'])) {
                    $query .= "type=";
                    $query .= $options['type'];
                    $query .= ",";
                }
                // pull off the last comma
                $length = strlen($query);
                $query = substr($query,0,$length-1);

                $query .= " WHERE id=" . $id . ";";

                //var_dump($query);
                
                // put is allowing any id to return success
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $result = true;
                } else {
                    $this->Log->logError("Error saving manager information");
                    $result = false;
                }
            }
        }
        return $result;
    }


    public function post($data) {
        global $Log;
        $this->Log = $Log;

        //var_dump($data);
        // requires a name to be entered
        if (!isset($data["name"])){
            $message = "No name in data. Please provide name.";
            $this->Log->logInfo($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            return $result;
        } else {
            $name = $data['name'];
        }
        $manager_id = isset($data['manager_id']) ? $data['manager_id'] : null;
        $type = isset($data['type']) ? $data['type'] : 0;

        $managerObj = $this->get(null, $name);
        if (is_array($managerObj) && isset($managerObj['managers'])) {
            $managers = $managerObj['managers'];
            if (count($managers) > 0) {
                $message = "Manager " . $name . " already exists.";
                $this->Log->logInfo($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            } else {
                // need to add a function if they have provider_ids
                $result = $this->addManager($name, $manager_id, $type);
            }
        } else {
            $message = "Manager in improper format.";
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
        }
        return $result;
    }

    private function addManager($name, $manager_id, $type) {
        $defaultManager = false;
        if ($manager_id === null) {
            $manager_id = 1;
            $defaultManager = true;
        }
        $id = false;
        $str = "INSERT INTO mgportal.managers (name, manager_id, type)";
        $str .= " VALUES (%s, %d, %d)";
        $str .= " RETURNING id;";
        $query = sprintf($str, Database::db_escape_string($name), $manager_id, $type);

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false) {
            $row = Database::db_fetch_row($this->Log,  $result);
            $id = $row[0];
            // set manager to self if not set.
            if ($defaultManager) {
                $query = "UPDATE mgportal.managers SET manager_id=" . $id . " WHERE id=" . $id . ";";
                $result = Database::db_query($this->conn, $this->Log, $query);
            }
        } else {
            $this->Log->logError("Error saving manager information");
        }
        return $id;
    }

    public function delete($id) {
        if ($id === null) {
            $result = array(
                'result' => false,
                'message' => 'Manager id is not optional for delete.'
            );
        } else if ($this->getReferences($id) !== false){
            $result = array(
                'result' => false,
                'message' => 'Manager has references (e.g., locations) that must be deleted first.'
            );
        } else {
            $query = "DELETE FROM mgportal.managers WHERE id=";
            $query .= $id;
            $query .= " RETURNING id;";
            $queryResult = Database::db_query($this->conn, $this->Log, $query);
            if ($queryResult !== false) {
                $row = Database::db_fetch_row($this->Log,  $queryResult);
                if ($row === null || $row === false) {
                    $result = array(
                        'result' => false,
                        'message' => "Error: manager id " . $id . " does not exist"
                    );
                } else {
                    $result = true;
                }
            } else {
                $message = "Error deleting manager " . $id;
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
        }
        return $result;
    }

    public function getDeviceChildren($id) {
        $result = array();
        // check for device children
        $query = "SELECT d.id, d.unique_identifier, d.hostname FROM mgportal.managers m INNER JOIN mgportal.locations l ON m.id=l.manager_id INNER JOIN mgportal.devices d ON l.id=d.location_id where m.id=" . $id;
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $device = array('id' => $row[0], 'uuid' => $row[1], 'hostname' => $row[2]);
                $result[] = $device;
            }
        }
        return $result;
    }

    private function addCredentials(&$item, $id, $name) {
        $credential_id = $id;

        // set defaults of no credentials and id 0.
        $item['credential_name'] = self::NO_CREDENTIALS;
        $item['credential_id'] = "0";

        if ($credential_id !== null && $credential_id !== "") {
            $item['credential_id'] = $credential_id;
            $credential_display_name = $name;
            if ($credential_display_name !== null && $credential_display_name !== "") {
                $item['credential_name'] = $credential_display_name;
            } else {
                $item['credential_name'] = self::UNNAMED_CREDENTIALS;
            }
        }
    }

    public function getAllDevices($manager_id = null, $parent_id = null, $isReport = false) {
        $result = array();
        // get managers with locations.
        $query = "SELECT DISTINCT ON (d.id) m.id, m.name, l.id, l.name, d.id, d.unique_identifier, d.hostname, d.activity, d.version";
        $query .= ", c.credential_id, c.display_name";
        if ($isReport) {
            $query .= ", l.street, l.city, l.state, l.zip, l.country, l.phone, l.longitude, l.latitude";
        }
        $query .= " FROM mgportal.managers m INNER JOIN mgportal.locations l ";
        $query .= "ON m.id=l.manager_id INNER JOIN mgportal.devices d ON l.id=d.location_id";
        $query .= " LEFT OUTER JOIN mgportal.credentials c ON c.credential_id=d.credential_id";
        $conjunction = " WHERE ";
        if ($manager_id != null) {
            $query .= " WHERE m.id=" . $manager_id;
            $conjunction = " OR ";
        }
        if ($parent_id !== null) {
            $query .=  $conjunction . "m.manager_id=" . $parent_id;
        }
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $item = array('manager_id' => $row[0],
                    'manager' => $row[1],
                    'location_id' => $row[2],
                    'location' => $row[3],
                    'device_id' => $row[4],
                    'uuid' => $row[5],
                    'hostname' => $row[6],
                    'activity' => $row[7],
                    'version' => $row[8]);

                $this->addCredentials($item, $row[9], $row[10]);

                if ($isReport) {
                    $locationDetail = array('street' => $row[11],
                        'city' => $row[12],
                        'state' => $row[13],
                        'zip' => $row[14],
                        'country' => $row[15],
                        'phone' => $row[16],
                        'longitude' => $row[17],
                        'latitude' => $row[18]);
                    $item = array_merge($item, $locationDetail);
                }
                $result[] = $item;
            }
        }
        return $result;
    }

    public function getAllLocations($manager_id = null, $parent_id = null)
    {
        $result = array();
        $query = "SELECT id, manager_id, name from mgportal.managers";
        $conjunction = " WHERE ";
        if ($manager_id != null) {
            $query .= " WHERE id=" . $manager_id;
            $conjunction = " OR ";
        }
        if ($parent_id !== null) {
            $query .=  $conjunction . "manager_id=" . $parent_id;
        }
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)) {
                $item = array('id' => $row[0],
                    'manager_id' => $row[1],
                    'manager' => $row[2]);
                $result[] = $item;
            }
        }
        if (count($result) > 0) {
            foreach ($result as &$manager) {
                $locations = array();
                $query = "SELECT id, name, country, street, city, state, zip, phone, latitude, longitude from mgportal.locations WHERE manager_id=" . $manager['id'];
                $qResult = Database::db_query($this->conn, $this->Log, $query);
                if ($qResult !== false) {
                    while ($row = Database::db_fetch_row($this->Log,  $qResult)) {
                        $location = array('location_id' => $row[0],
                            'location' => $row[1],
                            'country' => $row[2],
                            'street' => $row[3],
                            'city' => $row[4],
                            'state' => $row[5],
                            'zip' => $row[6],
                            'phone' => $row[7],
                            'latitude' => $row[8],
                            'longitude' => $row[9]);
                        $locations[] = $location;
                    }
                    $manager['locations'] = $locations;
                    $manager['locations_str'] = implode(',', array_map(function($location) { return $location['location']; }, $locations));
                }
                $query = "SELECT name from mgportal.managers where id=" . $manager['manager_id'];
                $qResult = Database::db_query($this->conn, $this->Log, $query);
                if ($qResult !== false) {
                    $row = Database::db_fetch_row($this->Log,  $qResult);
                    $parentName = $row[0];
                    $manager['parent'] = $parentName;
                }
            }
        }
        return $result;
    }

    public function getAllLocationsFlat($manager_id = null) {
        $query = "SELECT m.id, m.manager_id, m.name, l.id, l.name, l.country, l.street, l.city, l.state, l.zip, l.phone, l.latitude, l.longitude FROM mgportal.managers m";
        $query .= " INNER JOIN mgportal.locations l ON m.id=l.manager_id";
        if ($manager_id != null) {
            $query .= " WHERE m.id=" . $manager_id;
        }
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        $foundIDs = array();
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $foundIDs[] = $row[0];
                $item = array('id' => $row[0],
                    'manager_id' => $row[1],
                    'manager' => $row[2],
                    'location_id' => $row[3],
                    'location' => $row[4],
                    'country' => $row[5],
                    'street' => $row[6],
                    'city' => $row[7],
                    'state' => $row[8],
                    'zip' => $row[9],
                    'phone' => $row[10],
                    'latitude' => $row[11],
                    'longitude' => $row[12]);
                $result[] = $item;
            }
        }
        if (count($foundIDs) > 0) {
            $foundIDs = implode(",", $foundIDs);
            // get managers with no location.
            $query = "SELECT id, manager_id, name FROM mgportal.managers WHERE id NOT IN (" . $foundIDs . ")";
            if ($manager_id != null) {
                $query .= " AND id=" . $manager_id;
            }
        } else {
            $query = "SELECT id, manager_id, name FROM mgportal.managers";
            if ($manager_id != null) {
                $query .= " WHERE id=" . $manager_id;
            }
        }
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $item = array('id' => $row[0],
                    'manager_id' => $row[1],
                    'manager' => $row[2]);
                $result[] = $item;
            }
        }
        return $result;
    }

    public function getReferences($id){
        $result = array();

        // check for manager children
        $mChildren = $this->getChildren($id);
        if ($mChildren !== false && count($mChildren) > 0){
            $result[] = $mChildren;
        }

        // check for user children
        $query = "SELECT user_id, username FROM mgportal.users WHERE company_id=" . $id;
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false){
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $user = array('user_id' => $row[0], 'username' => $row[1]);
                $result[] = $user;
            }
        }

        // check for device children
        $deviceChildren = $this->getDeviceChildren($id);
        if ($deviceChildren !== false && count($deviceChildren) > 0){
            $result[] = $deviceChildren;
        }

        // check for location children
        $managerLocations = $this->getAllLocations($id);
        if (is_array($managerLocations) && count($managerLocations) > 0){
            $managerLocation = $managerLocations[0];
            if (isset($managerLocation['locations'])) {
                $locations = $managerLocation['locations'];
                foreach ($locations as $location) {
                    $result[] = $location;
                }
            }
        }

        if (count($result) == 0){
            $result = false;
        }

        return $result;
    }

    public function uploadImage($img){
        $target_dir ="/var/www/html/assets";

        if(isset($_POST["submit"])) {
            $check = getimagesize($_FILES["fileToUpload"]["name"]);
        }

        $query = "INSERT INTO mgportal.managers";
    }
};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $managers = new Managers($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        $isReport = isset($_GET['type']) && $_GET['type'] == 'report';
        $parent_id = isset($_GET['manager_id']) ? (int)$_GET['manager_id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                if (isset($_GET['text'])){
                    $text = $_GET['text'];
                    $result = $managers->getManagerType($which, $text);
                } elseif (isset($_GET['level'])) {
                    $level = $_GET['level'];
                    if ($level == 1) {
                        $result = $managers->getChildren($which);
                    } else if ($level == 'all') {
                        $result = $managers->getAllChildren($which);
                        $result = array("tree" => $result);
                    } else if ($level == 'devices') {
                        $result = $managers->getAllDevices($which, $parent_id, $isReport);
                        $result = $isReport ? array("data" => $result) : array("managers" => $result);
                    } else if ($level == 'locations') {
                        $result = $managers->getAllLocations($which, $parent_id);
                        $result = array("managers" => $result);
                    } else {
                        $result = array("message" => "Level invalid");
                    }
                } else {
                    $result = $managers->get($which, null);
                }
                $http->buildResult($result, "get");
                break;
            case 'put':
                $result = $managers->updateManager($which, $http->getData());
                $http->buildResult($result, "put");
                break;
            case 'post':
                $result = $managers->post($http->getData());
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $managers->delete($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}
?>