<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class RemoteREST extends BaseAPI {

    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function remoteAuthenticate($data, $remoteREST)
    {
        global $Log;

        if (isset($remoteREST['address'])) {
            $address = $remoteREST['address'];
            // ensure address is "clean"
            $address = trim($address);
        } else {
            $address = $remoteREST['ip'] . ':' . $remoteREST['port'];
        }

        $Log->writeVariable("redirect REST, authenticate");
        $curl = curl_init();
        $url = "https://" . $address . "/api/login";
        $credentials = array("username" => $data['username'], "password" => $data['password']);
        $credentials_string = json_encode($credentials);
        $Log->writeVariable("authenticating, url is $url");
        //$Log->writeVariable("authenticating, auth is $credentials_string");
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($curl, CURLOPT_POSTFIELDS, $credentials_string);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json')
        );
        $result = curl_exec($curl);
        $Log->writeVariable("after curl");
        $Log->writeVariable($result);
        if ($result === false) {
            $Log->writeVariable("the curl authentication request failed ");
            $curlError = curl_error($curl);
            $Log->writeVariable($curlError);
            if (curl_errno($curl) === CURLE_SSL_CONNECT_ERROR) {
                $curlError = "Error connecting to appliance.  Please click Manage to close any open tunnel, then Connect to re-open and re-connect.";
            }
            $result = array(
                'result' => false,
                'message' => $curlError
                );
        } else {
            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $Log->writeVariable('curl request: http code');
            $Log->writeVariable($httpcode);

            // decode JSON string into php associative array.
            $result = json_decode($result, true);

            if ($httpcode !== HTTP::CREATED) {
                if (isset($result['result']) && isset($result['result'][0]) && isset($result['result'][0]['message'])) {
                    $result['message'] = "Error authenticating on remote appliance: " . $result['result'][0]['message'];
                } else {
                    $result['message'] = "Unknown error authenticating on remote appliance.";
                }
                $result['result'] = false;
            } else {
                $result['result'] = true;
            }
        }
        curl_close($curl);
        return $result;
    }

    public function runRemote($data, $remoteREST) {
        $curl = curl_init();
        global $Log;

        $address = $remoteREST['address'];
        // ensure address is "clean"
        $address = trim($address);
        $url = $remoteREST['url'];
        $auth_token = $remoteREST['auth'];
        $method = $remoteREST['method'];

        $url = "https://" . $address . $url;
        $Log->writeVariable("remote REST, url is $url");

        // prepare the data to send with curl
        $encoded_data = json_encode($data);

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        if ($method !== "GET") {
            curl_setopt($curl, CURLOPT_POSTFIELDS, $encoded_data);
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'AuthToken: ' . $auth_token
            )
        );
        $result = curl_exec($curl);
        $Log->writeVariable("after curl");
        $Log->writeVariable($result);
        if ($result === false) {
            $Log->writeVariable("remoteRequest:the curl request failed ");
            $Log->writeVariable(curl_error($curl));
        } else {
            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $Log->writeVariable('curl request: http code');
            $Log->writeVariable($httpcode);

            // decode JSON string into php associative array.
            $result = json_decode($result, true);

            if ($httpcode !== HTTP::CREATED && $httpcode !== HTTP::OK) {
                if (isset($result['result']) && isset($result['result'][0]) && isset($result['result'][0]['message'])) {
                    $result['message'] = "Error running command on remote appliance: " . $result['result'][0]['message'];
                } else {
                    $result['message'] = "Unknown error running command on remote appliance.";
                }
                $result['result'] = false;
            } else {
                $result['result'] = true;
            }
        }
        curl_close($curl);

        return $result;

    }

};


$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $rest = new RemoteREST($conn, $Log);
        $remoteREST = array();
        $data = $http->getData();
        if (isset($data['remoteREST'])) {
            $remoteREST = $data['remoteREST'];
        }
        switch ($http->getMethod()) {
            case 'post':
                $cmd = isset($_GET['cmd']) ? $_GET['cmd'] : null;
                if ($cmd === 'login') {
                    $result = $rest->remoteAuthenticate($data, $remoteREST);
                } else {
                    $result = $rest->runRemote($data, $remoteREST);
                }
                $http->buildResult($result, "post");
                break;
            case 'put':
                $result = $rest->runRemote($data, $remoteREST);
                $http->buildResult($result, "put");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}


?>

