<?php
/**
 *  Status API
 */

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP($Config->getDeviceAllowance());

class Status extends BaseAPI {
    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }
    
    public function get($device_id, $company_id, $parent_id, $getCounts)
    {

        $deviceArray = array();
        $devices = $this->getDevicesDetailed($device_id, $company_id, $parent_id);
        foreach ($devices as $device) {
            $devID = $device['id'];
            if ($getCounts) {
                $backups = $this->getBackupCountsDays($devID);
                $alerts = $this->getAlertCounts($devID);
                $alerts['timestamp'] = $this->getTimestamp("alerts", $devID);
                $alerts['updated_date'] = date(Status::DATE_TIME_FORMAT_US, $alerts['timestamp']);
                $copies = $this->getBackupCountsDays($devID, "copies");
                $assets = $this->getAssetCounts($devID);
            } else {
                $backups = $this->getBackups($devID);
                $alerts = $this->getAlerts($devID);
                $copies = $this->getBackups($devID, "copies");
                $assets = array();
            }
            $device['backups'] = $backups;
            $device['alerts'] = $alerts;
            $device['copies'] = $copies;
            $device['assets'] = $assets;
            $deviceArray[] = $device;
        }

        return array('devices' => $deviceArray);
    }

    private function getDevices() {
        $devices = array();

        $query = "SELECT id, unique_identifier, hostname, activity FROM mgportal.devices";
        $devicesDB = Database::db_query($this->conn, $this->Log, $query);
        if ($devicesDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $devicesDB)) {
                $device = array();
                $device['id'] = $row[0];
                $device['uuid'] = $row[1];
                $device['name'] = $row[2];
                $device['activity'] = $row[3];
                $devices[] = $device;
            }
        }
        return $devices;
    }

    public function getDevicesDetailed($device_id = null, $company_id = null, $parent_id = null) {
        $result = array();
        $query = "SELECT DISTINCT on (d.id) d.id, d.hostname, d.unique_identifier, d.activity, l.id, l.name, m.id, m.name FROM mgportal.devices d INNER JOIN mgportal.locations l ON d.location_id=l.id ";
        $query .= "INNER JOIN mgportal.managers m ON l.manager_id=m.id";
        if ($device_id != null) {
            $query .= " AND d.id=" . $device_id;
        }
        $conjunction = " AND ";
        if ($company_id != null) {
            $query .= " AND m.id=" . $company_id;
            $conjunction = " OR ";
        }
        if ($parent_id !== null) {
            $query .= $conjunction . " m.manager_id=" . $parent_id;
        }
        $query .= ";";
        $this->Log->logDebug("Devices Detailed query " . $query);
        $qResult = Database::db_query($this->conn, $this->Log, $query);
        if ($qResult !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $qResult)){
                $item = array('id' => $row[0],
                    'name' => $row[1],
                    'uuid' => $row[2],
                    'activity' => $row[3],
                    'location_id' => $row[4],
                    'location_name' => $row[5],
                    'manager_id' => $row[6],
                    'manager_name' => $row[7]);
                $result[] = $item;
            }
        }
        return $result;
    }

    private function getBackups($device_id = null, $tableName = "backups") {
        $backups = array();

        $query = "SELECT client_name, app_name, database_name, instance_name, type, status, complete, start_date, end_date, elapsed_time FROM mgportal." . $tableName;
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "';";
        }

        $backupsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($backupsDB !== false) {
           while ($row = Database::db_fetch_row($this->Log,  $backupsDB)) {
               $backup = array();
               $backup['client_name'] = $row[0];
               $backup['app_name'] = $row[1];
               $backup['database_name'] = $row[2];
               $backup['instance_name'] = $row[3];
               $backup['type'] = $row[4];
               $backup['status'] = $row[5];
               $backup['complete'] = $row[6] == 1;
               $backup['start_date'] = $row[7];
               $backup['end_date'] = $row[8];
               $backup['elapsed_time'] = $row[9];
               $backups[] = $backup;
           }
        }
        return $backups;
    }

    private function getBackupCounts($device_id = null, $tableName = "backups") {
        $backupTotal = 0;
        $successTotal = 0;
        $failedTotal = 0;
        $warnTotal = 0;
        $activeTotal = 0;

        $query = "SELECT count(*) FROM mgportal." . $tableName;
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND status='Success';";
        } else {
            $query .= " WHERE status='Success';";
        }
        $backupsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($backupsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $backupsDB)) {
                $successTotal = (int)$row[0];
            }
        }

        $query = "SELECT count(*) FROM mgportal." . $tableName;
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND status='Failure';";
        } else {
            $query .= " WHERE status='Failure';";
        }
        $backupsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($backupsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $backupsDB)) {
                $failedTotal = (int)$row[0];
            }
        }

        $query = "SELECT count(*) FROM mgportal." . $tableName;
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND status='Warning';";
        } else {
            $query .= " WHERE status='Warning';";
        }
        $backupsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($backupsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $backupsDB)) {
                $warnTotal = (int)$row[0];
            }
        }


        $query = "SELECT count(*) FROM mgportal." . $tableName;
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND status='Running';";
        } else {
            $query .= " WHERE status='Warning';";
        }
        $backupsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($backupsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $backupsDB)) {
                $activeTotal = (int)$row[0];
            }
        }

        $backupTotal = $successTotal + $failedTotal + $warnTotal + $activeTotal;

        $backups = array(
            'total' => array('count' => $backupTotal),
            'successes' => array('count' => $successTotal),
            'failures' => array('count' => $failedTotal),
            'active' => array('count' => $activeTotal),
            'warnings' => array('count' => $warnTotal)
        );
        return $backups;
    }

    private function getAlerts($device_id = null) {
        $alerts = array();

        $query = "SELECT id, created, updated, resolved, severity, source_id, source_name, message FROM mgportal.alerts";
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "';";
        }

        $alertsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($alertsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $alertsDB)) {
                $alert = array();
                $alert['id'] = $row[0];
                $alert['created'] = $row[1];
                $alert['updated'] = $row[2];
                $alert['resolved'] = $row[3] == 1;
                $alert['severity'] = $row[4];
                $alert['source_id'] = $row[5];
                $alert['source_name'] = $row[6];
                $alert['message'] = $row[7];
                $alerts[] = $alert;
            }
        }
        return $alerts;
    }

    private function getBackupCountsDays($device_id = null, $tableName = "backups") {
        $data = array();
        for ($i = 0; $i < 7; $i++) {
            $data[$i] = array();
        }

        $query = "SELECT start_date, status FROM mgportal." . $tableName;
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "';";
        }
        $backupsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($backupsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $backupsDB)) {
                $thisDate = $row[0];
                $status = $row[1];
                $dayOfWeek = date('w', strtotime($thisDate));
                $this->Log->logInfo("this Date and status");
                $message = sprintf("%s %s %d", $thisDate, $status, $dayOfWeek);
                $this->Log->logInfo($message);
                $data[$dayOfWeek][] = $status;
            }
        }

        $backupsArray = array();
        foreach ($data as $dow => $statusDay) {
            $successTotal = 0;
            $failedTotal = 0;
            $warnTotal = 0;
            $activeTotal = 0;
            foreach ($statusDay as $status) {
                $successTotal += $status === 'Success';
                $failedTotal += $status === 'Failure';
                $warnTotal += $status === 'Warning';
                $activeTotal += $status === 'Running';
            }
            $backupTotal = $successTotal + $failedTotal + $warnTotal + $activeTotal;
            $backups = array(
                'day' => $dow,
                'total' => array('count' => $backupTotal),
                'successes' => array('count' => $successTotal),
                'failures' => array('count' => $failedTotal),
                'active' => array('count' => $activeTotal),
                'warnings' => array('count' => $warnTotal)
            );
            $backupsArray[] = $backups;
        }
        if (count($backupsArray) > 0) {
            $info = $this->getTimestampsInfo($tableName, $device_id);
            if ($info !== false) {
                $timestamp = $info['timestamp'];
                $day_count = $info['day_count'];
            } else {
                $timestamp = 0;
                $day_count = 0;
            }
            $backupsArray[0]['updated_date'] = date(Status::DATE_TIME_FORMAT_US, $timestamp);
            $backupsArray[0]['timestamp'] = $timestamp;
            $backupsArray[0]['updated_days'] = $day_count;
        }

        return $backupsArray;
    }

    private function getAlertCounts($device_id = null) {

        $alertsTotal = 0;
        $openTotal = 0;
        $criticalOpen = 0;
        $criticalResolved = 0;

        $query = "SELECT count(*) FROM mgportal.alerts";
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "';";
        }
        $alertsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($alertsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $alertsDB)) {
                $alertsTotal = (int)$row[0];
            }
        }

        $query = "SELECT count(*) FROM mgportal.alerts";
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND resolved=0;";
        } else {
            $query .= " WHERE resolved=0;";
        }
        $alertsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($alertsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $alertsDB)) {
                $openTotal = (int)$row[0];
            }
        }

        $query = "SELECT count(*) FROM mgportal.alerts";
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND resolved=0 AND severity='critical';";
        } else {
            $query .= " WHERE resolved=0 AND severity='critical';";
        }
        $alertsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($alertsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $alertsDB)) {
                $criticalOpen = (int)$row[0];
            }
        }

        $query = "SELECT count(*) FROM mgportal.alerts";
        if ($device_id !== null) {
            $query .= " WHERE device_id='" . $device_id . "' AND resolved=1 AND severity='critical';";
        } else {
            $query .= " WHERE resolved=1 AND severity='critical';";
        }
        $alertsDB = Database::db_query($this->conn, $this->Log, $query);
        if ($alertsDB !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $alertsDB)) {
                $criticalResolved = (int)$row[0];
            }
        }

        $alerts = array(
            'total' => $alertsTotal,
            'open' => array(
                'total' => $openTotal,
                'critical' => $criticalOpen
            ),
            'resolved' => array(
                'total' => $alertsTotal - $openTotal,
                'critical' => $criticalResolved
            )
        );
        return $alerts;
    }

    private function getCopies($device_id = null) {
        return array();
    }

    private function getAssetCounts($device_id) {
        $assets = array();

        $query = "SELECT date, vm_count, server_count, app_count, gb_used, vm_replicated, all_replicated, replication_used";
        $query .= " FROM mgportal.protected where device_id='";
        $query .= $device_id;
        $query .= "' ORDER BY date DESC LIMIT 1";
        $protected = Database::db_query($this->conn, $this->Log, $query);
        if ($protected !== false) {
            while ($item = Database::db_fetch_row($this->Log,  $protected)) {
                $timestamp = $item[0];
                $dateString = date('m/d/Y', $timestamp);

                $vm_protected = (int)$item[1];
                $server_protected = (int)$item[2];
                $app_protected = (int)$item[3];
                $gb_used = (float)$item[4];
                $vm_replicated = (int)$item[5];
                $all_replicated = (int)$item[6];
                $replication_used = (float)$item[7];

                $assets = array("date" => $dateString,
                    "vms" => $vm_protected,
                    "servers" => $server_protected,
                    "apps" => $app_protected,
                    "gb_protected" => $gb_used,
                    "replication_protected" => $replication_used,
                    "vms_replicated" => $vm_replicated,
                    "all_replicated" => $all_replicated);
            }
        }

        return $assets;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $status = new Status($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        $company = isset($_GET['company_id']) ? $_GET['company_id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $counts = isset($_GET['counts']) && $_GET['counts'] == 1;
                $parent_id = isset($_GET['manager_id']) ? (int)$_GET['manager_id'] : null;
                $result = $status->get($which, $company, $parent_id, $counts);
                if ($result !== false) {
                    $http->buildSuccess($result);
                } else {
                    $http->buildFailure(false, "Error getting status");;
                }
                break;
            case 'put':
                // $which is device ID, send command to update summaries.
                $cmd = isset($_GET['cmd']) ? $_GET['cmd'] : Configuration::CMD_GET_ALL;
                $result = $status->sendCommand($which, $cmd);
                $http->buildResult($result, "put");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $message = $http->getError();
    $message = $message == "" ? $authMessage : $message;
    $Log->logError($message);
    $http->buildFailure(false, $message);
}
?>