<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger();
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile("portal_api");
$http = new HTTP($Config->getDeviceAllowance());

class Tunnel extends BaseAPI {

    public function __construct($conn, $Log){
        parent::__construct($conn, $Log);
    }

    public function get($which){
        $key = "";
        switch ($which) {
            case "manager-key":
                $key = file_get_contents('/etc/ssh/ssh_host_rsa_key.pub');
                break;
        }
        return array("data" => $key);
    }

    public function addKey($data){
        $result = true;
        global $Log;
        if (isset($data['data'])) {
            $keyData = $data['data'];
            $Log->logError("key data is $keyData");
            $tempName = tempnam('/tmp', 'keyFile');
            $returnValue = file_put_contents($tempName, $keyData, FILE_APPEND);
            if ($returnValue !== false) {
                system('chmod 666 ' . $tempName, $returnValue);
                //$password = Commands::getPortalPassword();
                //$cmd = "echo " . $password . "| sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh authorizedKey " . $tempName;
                $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh authorizedKey " . $tempName;
                system($cmd, $returnValue);
                $Log->logError("return of authorize key is " . $returnValue);
                system('rm -f ' . $tempName, $returnValue);
                $Log->logError("return of rm temp file is " . $returnValue);
            } else {
                $result = array('result' => false, 'message' => 'error saving key file to temporary location');
            }
        } else {
            $Log->logError("key data not in valid format");
            $result = array('result' => false, 'message' => 'authorized key is not valid');
        }
        return $result;
    }

    public function sendCommand($deviceid, $cmd, $parameter = null) {

        if ($cmd === Configuration::CMD_OPEN_TUNNEL) {
            $result = $this->checkTunnelPorts();
            if ($result !== true) {
                return $result;
            }
        }

        $result = parent::sendCommand($deviceid, $cmd);

        if ($cmd === Configuration::CMD_OPEN_TUNNEL) {
            if ($result['result'] == true) {
                $tunnel = $result['tunnel'];
                $host = $result['host'];
                //$password = Commands::getPortalPassword();
                //$cmd = "sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh loopBack " . $tunnel . " " . $host;
                $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh loopBack " . $tunnel . " " . $host;
                $this->Log->logNotice("host tunnel cmd: " . $cmd);
                //$cmd = "echo " . $password . " | " . $cmd;
                $hostTunnelOutput = shell_exec($cmd);
                $result['host_tunnel'] = $hostTunnelOutput;
            }
        } else if ($cmd === Configuration::CMD_CLOSE_TUNNEL) {
            $tunnel = $parameter;
            //$password = Commands::getPortalPassword();
            //$cmd = "sudo -u mgportal -S /usr/local/mgportal/bin/tunnel.sh loopBackClose " . $tunnel;
            $cmd = "sudo -u mgportal /usr/local/mgportal/bin/tunnel.sh loopBackClose " . $tunnel;
            $this->Log->logNotice("host tunnel cmd: " . $cmd);
            //$cmd = "echo " . $password . " | " . $cmd;
            $hostTunnelOutput = shell_exec($cmd);
        }

        return $result;
    }

    public function deleteKey($which){
        $result = true;
        return $result;
    }

    /*
     * Checks to see if we can open a local tunnel port.  Returns true if so or an error mesage array if not.
     */
    private function checkTunnelPorts() {
        $result = true;
        $portCount = $this->getPortCount();
        $this->Log->logDebug("Port Count = " . $portCount);
        if ($portCount > 0) {
            $portsUsed = shell_exec("ps -eo args | grep mgportal | grep NoHostAuthentication | grep -v grep | wc -l");
            $portsUsed = (int)$portsUsed;
            $this->Log->logDebug("Ports Used = " . $portsUsed);
            if ($portsUsed >= $portCount) {
                $result = array('result' => false, 'message' => 'There are ' . $portsUsed . ' management ports used out of ' . $portCount . ' available.  ' .
                    'You need to either close a connection or increase the port count');
                }
        } else {
            $result = array('result' => false, 'message' => 'The port count on the server is 0.  Please increase the DEM port count on the server to allow a connection to be created.');
        }
        return $result;
    }

    private function getPortCount() {
        $portCount = 0;

        if (function_exists("bp_get_ini_value")) {
            // If BPL APIs are defined, use them to read the master.ini.
            bp_bypass_cookie(2);
            $ports = bp_get_ini_value("CMC", "DEMPortCount");
            if ($ports !== false) {
                $portCount = (int)$ports;
            } else {
                $this->Log->logError("Cannot get port count" . bp_error());
            }
        } else {
            global $Config;
            $portCount = $Config->getPortCount();
        }
        return $portCount;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $tunnel = new Tunnel($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        $cmd = isset($_GET['cmd']) ? $_GET['cmd'] : null;
        switch ($http->getMethod()) {
            case 'get':
                if ($cmd == $Config::CMD_GET_KEY) {
                    $result = $tunnel->get('manager-key');
                }
                $http->buildResult($result, "get");
                break;
            case 'post':
                $result = $tunnel->addKey($http->getData());
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $type = isset($_GET['type']) ? $_GET['type'] : null;
                $result = true;
                if ($type == 'key') {
                    $result = $tunnel->deleteKey($which);
                }
                $http->buildResult($result, "delete");
                break;
            case 'put':
                // $which is device ID.
                // an optional argument is needed for some commands (tunnel number when closing)
                $parameter = isset($_GET['parameter']) ? $_GET['parameter'] : null;
                $result = $tunnel->sendCommand($which, $cmd, $parameter);
                $http->buildResult($result, "put");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}
?>
