<?php
/**
 *  Users API
 */


const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");
require_once("./system/" . "password.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Users extends BaseAPI {
    const MIN_PASSWORD_LENGTH = 8;

    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    public function get($which, $name) {
        $userSession = $this->getSessionInfo();
        if ($userSession === false) {
            return false;
        }

        $query = "SELECT DISTINCT ON (users.user_id) users.user_id, users.username, users.privileges, managers.name, users.email_address, managers.id, managers.manager_id FROM mgportal.users";
        $query .= " JOIN mgportal.managers on (users.company_id = managers.id)";

        switch ($userSession['privilege']) {
            case 0:
                // monitor, current user only.
                $query .= " WHERE user_id=" . $userSession['user_id'];
                break;
            case 1:
                if ($which !== null) {
                    $query .= " WHERE user_id=" . $which . " AND ";
                } else if ($name !== null) {
                    $query .= " WHERE username='" . $name . "' AND ";
                } else {
                    $query .= "WHERE ";
                }
                // manager, show all users of this company and managed companies.
                $query .= " manager_id=" . $userSession['company_id'] . " OR company_id=" . $userSession['company_id'];
                break;
            case 2:
            case 3:
                // superuser and administrator, all or selective.
                if ($which !== null) {
                    $query .= " WHERE user_id=" . $which;
                } else if ($name !== null) {
                    $query .= " WHERE username='" . $name . "'";
                }
                $query .= ";";
                break;
        }

        $query .= ";";
        $users = array();

        $dbUsers = Database::db_query($this->conn, $this->Log, $query);
        if ($dbUsers !== false) {
            while ($row = Database::db_fetch_row($this->Log,  $dbUsers)) {
                //printf("asset = %s\n", $row[1]);
                $id = $row[0];
                $userName = $row[1];
                $userPrivileges = $row[2];
                $companyName = $row[3];
                $email_address = $row[4];
                $company_id = $row[5];
                $managers_manager_id = $row[6];
                $user = array('user_id' => $id, 'username' => $userName, 'privilege' => $userPrivileges,
                              'email_address' => $email_address, 'company' => $companyName,
                               'company_id' => $company_id, 'managers_manager' => $managers_manager_id );
                $users[] = $user;
            }
        }
        return array("users" => $users);
    }

    private function addUser($userName, $password, $privilege, $company_id = null, $email_add = null) {
        $user_id = false;
        global $Log;
        if ($company_id == null && $email_add == null) {
            $Log->logError("here, company id and email are null");
            $str = "INSERT INTO mgportal.users (username, password, privileges)";
            $str .= " VALUES ('%s', '%s', %d)";
            $str .= " RETURNING user_id;";
            $query = sprintf($str, $userName, $password, $privilege);
        } else if ($company_id !== null && $email_add == null) {
            $Log->logError("here company is not null but email is");
            $str = "INSERT INTO mgportal.users (username, password, privileges, company_id)";
            $str .= " VALUES ('%s', '%s', %d, %d)";
            $str .= " RETURNING user_id;";
            $query = sprintf($str, $userName, $password, $privilege, $company_id);
        } else if ($company_id == null && $email_add !== null){
            $Log->logError("here company is null but email isnt");
            $str = "INSERT INTO mgportal.users (username, password, privileges, email_address)";
            $str .= " VALUES ('%s', '%s', %d, '%s')";
            $str .= " RETURNING user_id;";
            $query = sprintf($str, $userName, $password, $privilege, $email_add);
        } else {
            $str = "INSERT INTO mgportal.users (username, password, privileges, company_id, email_address)";
            $str .= " VALUES ('%s', '%s', %d, %d, '%s')";
            $str .= " RETURNING user_id;";
            $query = sprintf($str, $userName, $password, $privilege, $company_id, $email_add);
        }
        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false) {
            $row = Database::db_fetch_row($this->Log,  $result);
            $user_id = $row[0];
        } else {
            $Log->logError("Error saving user information");
        }
        return $user_id;
    }

    public function updateUser($user_id, $options) {
        $result = false;
        
        if ($user_id === null) {
            $result = array(
                'result' => false,
                'message' => 'User id is not optional for update.'
            );
        } else if (isset($options['password']) && !$this->isValidPasswordFormat($options['password'])) {
            $result = array(
                'result' => false,
                'message' => "Passwords must be a minimum of 8 characters long and contain no spaces."
            );
        } else {
            if (is_array($options)) {
                $query = "UPDATE mgportal.users SET ";
                if (isset($options['password'])) {
                    $password = password_hash($options['password'], PASSWORD_DEFAULT);
                    $query .= "password='" . $password . "'";
                }
                if (isset($options['company_id'])) {
                    if (isset($password)) {
                        $query .= ", ";
                    }
                    $company_id = $options['company_id'];
                    $query .= "company_id=" . $company_id;
                }
                if (isset($options['email_address'])){
                    if (isset($password) || isset($company_id)){
                        $query .= ", ";
                    }
                    $email_address = $options['email_address'];
                    $query .= "email_address='" . $email_address . "'";
                }
                if (isset($options['privilege'])){
                    if (isset($password) || isset($company_id) || isset($email_address)){
                        $query .= ", ";
                    }
                    $priv = $options['privilege'];
                    $query .= "privileges=" . $priv;
                }

                $query .= " WHERE user_id='" . $user_id . "';";
                //var_dump($query);
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $result = true;
                } else {
                    $this->Log->logError("Error saving user information");
                }
            }          
        }
        return $result;
    }

    public function checkPrivilege($userid){
        $query = "SELECT privileges FROM mgportal.users WHERE userid=";
        $query .= $userid . ";";

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $this->conn, $query);

            if ($row === null){
                $result = array(
                    'result' => false,
                    'message' => "Could not find the user's privilege"
                );
            } else {
                $result = true;
            }
        } else {
            $message = "Error finding privilege for " . $userid;
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
        }
        return $result;
    }


    public function post($data) {
        $userName = $data['username'];
        $password = $data['password'];
        if ($this->isValidNameFormat($userName)) {
            if ($this->isValidPasswordFormat($password)) {
                $password = password_hash($data["password"], PASSWORD_DEFAULT);
                $company_id = isset($data['company_id']) ? $data['company_id'] : null;
                $email_address = isset($data['email_address']) ? $data['email_address'] : null;
                $privilege = isset($data['privilege']) ? $data['privilege'] : "0";

                $userObj = $this->get(null, $userName);
                if (is_array($userObj) && isset($userObj['users'])) {
                    $users = $userObj['users'];
                    if (count($users) > 0) {
                        $message = "User " . $userName . " already exists.";
                        $this->Log->logInfo($message);
                        $result = array(
                            'result' => false,
                            'message' => $message
                        );
                    } else {
                        $result = $this->addUser($userName, $password, $privilege, $company_id, $email_address);
                    }
                } else {
                    $message = "User list in improper format.";
                    $this->Log->logError($message);
                    $result = array(
                        'result' => false,
                        'message' => $message
                    );
                }
            } else {
                $message = "Passwords must be a minimum of 8 characters long and contain no spaces.";
                $result = array(
                    'result' => false,
                    'message' => $message
                );
            }
        } else {
            $message = "Usernames must be alphanumeric or include dot, dash, or @ characters.";
            $result = array(
                'result' => false,
                'message' => $message
            );
        }

        return $result;
    }

    private function isValidNameFormat($value) {
        $valid = true;
        $match = preg_match('/^[a-zA-Z0-9@_.\-]+$/i' ,$value);
        global $Log;
        $Log->writeVariable("MATCH: " . $match);
        if ($match === 0 || $match === false) {
            $valid = false;
        }
        return $valid;
    }

    private function isValidPasswordFormat($value) {
        $valid = true;
        $value = trim($value);
        if (strlen($value) < self::MIN_PASSWORD_LENGTH) {
            $valid = false;
        } else if (strchr($value, " ")) {
            $valid = false;
        }
        return $valid;
    }

    public function delete($user_id) {
        if ($user_id === null) {
            $result = array(
                'result' => false,
                'message' => 'User id is not optional for delete.'
            );
        } else {
            $userSession = $this->getSessionInfo();
            if ($userSession === false) {
                return false;
            }

            if ($user_id === $userSession['user_id']) {
                $result = array(
                    'result' => false,
                    'message' => 'You cannot delete yourself.'
                );
            } else {
                // save username
                $userName = $this->getNameFromID($user_id);
                $query = "DELETE FROM mgportal.users WHERE user_id=";
                $query .= $user_id;
                $query .= " RETURNING user_id;";
                $result = Database::db_query($this->conn, $this->Log, $query);
                if ($result !== false) {
                    $row = Database::db_fetch_row($this->Log,  $result);
                    if ($row == null) {
                        $result = array(
                            'result' => false,
                            'message' => "Error: user id " . $user_id . " does not exist"
                        );
                    } else {
                        $session = new Sessions($this->conn, $this->Log);
                        $result = $session->deleteUserSessions($user_id);
                        // Now delete any preferences for this user.
                        if (is_string($userName)) {
                            $this->deletePreferences($userName);
                        }
                    }
                } else {
                    $message = "Error deleting user " . $user_id;
                    $this->Log->logError($message);
                    $result = array(
                        'result' => false,
                        'message' => $message
                    );
                }
            }
        }
        return $result;
    }

    private function getNameFromID($userid){
        $query = "SELECT username FROM mgportal.users WHERE userid=";
        $query .= $userid . ";";

        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false){
            $row = Database::db_fetch_row($this->Log,  $this->conn, $query);
            if ($row === null){
                $result = array(
                    'result' => false,
                    'message' => "Could not find the user's name"
                );
            } else {
                $result = $row[0];
            }
        } else {
            $message = "Error finding name for " . $userid;
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
        }
        return $result;
    }

    private function deletePreferences($userName) {
        $query = "DELETE FROM mgportal.preferences WHERE ";
        $query .= " username='" . $userName . "';";
        $this->Log->logDebug("Query is " . $query);
        $result = Database::db_query($this->conn, $this->Log, $query);
        if ($result !== false) {
            $result = true;
        } else {
            $this->Log->logDebug("Error deleting all user preferences.");
        }
    }

    private function getSessionInfo() {
        global $http;
        $session = new Sessions($this->conn, $this->Log);
        $userSession = $session->getUserSessionForToken($http->getAuthToken());
        if ($userSession === false) {
            $this->Log->logError("can't get user info for token, error");
        }
        return $userSession;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $users = new Users($conn, $Log);
        $which = isset($_GET['id']) ? $_GET['id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $result = $users->get($which, null);
                //$result = $users->forgotPassword(1);
                $http->buildResult($result, "get");
                break;
            case 'put':
                $result = $users->updateUser($which, $http->getData());
                $http->buildResult($result, "put");
                break;
            case 'post':
                $result = $users->post($http->getData());
                $http->buildResult($result, "post");
                break;
            case 'delete':
                $result = $users->delete($which);
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}

function getRandString(){
    // create random
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $randstring = '';
    for ($i = 0; $i < 5; $i++) {
        $randstring .= $characters[rand(0, strlen($characters)-1)];
    }

    return $randstring;
}
?>